(***************************************************************************)
(*                                                                         *)
(*          UNIDAD PARA LA RESOLUCION GRAFICA DE PROBLEMAS LINEALES        *)
(*                      ELCHE 1 OCTUBRE DE 1992                            *)
(*                  AUTOR: FRANCISCO JOSE MORA LIZAN                       *)
(*                                                                         *)
(*                      ES LLAMADA DESDE PL.PAS                            *)
(*                                                                         *)
(***************************************************************************)

UNIT RG;

interface

USES Graph,CRT,DRIVERS,FONTS,TIPOS,Utiles;
Procedure Resolucion_Grafica;

implementation

{ Definicin de parmetros }
CONST
{
color[0]=color de la recta objetivo
color[1]=color de la restriccin 1
color[2]=color de la restriccin 2
color[3]=color de la restriccin 3
color[4]=color del ptimo
}
color:Array [0..4] OF INTEGER=(10,9,12,14,11);

{ Definicin de tipos utilizados internamente en esta unidad }
Type
 tipo_punto=RECORD
             x1,x2:REAL;
            END;
 tipo_recta=RECORD
             extremo1,extremo2:tipo_punto;
            END;

{ Definicin de varibles internas globales de esta unidad }
VAR
  { Variables utlizadas para la configuracin de la tarjeta de video}
  DriverGraf : Integer;
  ModoGraf   : Integer;
  CadenaTest : String;

  { Variables asociadas a la forma de las rectas y rango de representacin en pantalla }
  Rectas:ARRAY [0..3] OF TIPO_RECTA;
  MaxX1,MaxX2,CentroPantalla:REAL;
  GetMaxYY:Integer;

  { Variables para clculo de datos}
  numero_restricciones,numero_variables:Integer;
  puntos_de_corte:ARRAY[1..30] OF TIPO_PUNTO;
  numero_puntos_de_corte:INTEGER;
  OptimoX1,OptimoX2,OptimoX1_aux,OptimoX2_aux,MinMax:Real;{Guardan el valor de ptimo de Z,x1 y x2}
  Encontrado:BOOLEAN;

  { Variables auxiliares }
  i,j:REAL;
  n,m:INTEGER;
  tecla:CHAR;
  cadena_aux:STRING;{Variable intermedia para escribir nmeros en modo grfico}
  Pl_aux:Tipo_Problema_lineal;
  Es_optimo_recta:BOOLEAN;

{ Procedimientos para la gestin de la tarjeta de video }

Procedure Abort(Msg : string);
begin
  Writeln(Msg, ': ', GraphErrorMsg(GraphResult));
  Halt(1);
end;

Procedure Inicializar_Graficos;
BEGIN
 { Register all the drivers }
 if RegisterBGIdriver(@CGADriverProc) < 0 then
   Abort('CGA');
 if RegisterBGIdriver(@EGAVGADriverProc) < 0 then
   Abort('EGA/VGA');
 if RegisterBGIdriver(@HercDriverProc) < 0 then
   Abort('Herc');
 if RegisterBGIdriver(@ATTDriverProc) < 0 then
   Abort('AT&T');
 if RegisterBGIdriver(@PC3270DriverProc) < 0 then
   Abort('PC 3270');

 { Register all the fonts }
 if RegisterBGIfont(@GothicFontProc) < 0 then
   Abort('Gothic');
 if RegisterBGIfont(@SansSerifFontProc) < 0 then
   Abort('SansSerif');
 if RegisterBGIfont(@SmallFontProc) < 0 then
   Abort('Small');
 if RegisterBGIfont(@TriplexFontProc) < 0 then
   Abort('Triplex');

 DriverGraf := Detect;  { BGI determina la tarjeta grfica que se usa }
 InitGraph(DriverGraf,ModoGraf,'');
 IF GraphResult > 100 THEN
  BEGIN
   Writeln('>>Detenido por error en grficos: ',GraphErrorMsg(Graphresult));
   Halt(2)
  END;
End;

{ Procedimientos propios de la unidad}

{ Procedimiento para establecer las rectas}
Procedure Establecer_Rectas;
Begin
 {Inicializacin vector rectas}
 FOR n:=1 TO 3 DO
  Begin
   { clculo punto 1 }
   Rectas[n].extremo1.x1:=0;
   If (Pl_aux.restriccion[n].a[2]<>0) Then
    Begin
     Rectas[n].extremo1.x2:=Pl_aux.restriccion[n].d/Pl_aux.restriccion[n].a[2];
     numero_puntos_de_corte:=numero_puntos_de_corte+1;
     puntos_de_corte[numero_puntos_de_corte].x1:=0;
     puntos_de_corte[numero_puntos_de_corte].x2:=Rectas[n].extremo1.x2;
    End
   Else
    Rectas[n].extremo1.x2:=0;

   { clculo punto 2 }
   Rectas[n].extremo2.x2:=0;
   If (Pl_aux.restriccion[n].a[1]<>0) Then
    Begin
     Rectas[n].extremo2.x1:=Pl_aux.restriccion[n].d/Pl_aux.restriccion[n].a[1];
     numero_puntos_de_corte:=numero_puntos_de_corte+1;
     puntos_de_corte[numero_puntos_de_corte].x1:=Rectas[n].extremo2.x1;
     puntos_de_corte[numero_puntos_de_corte].x2:=0;
    End
   Else
    Rectas[n].extremo2.x1:=0;
  End;
End;

{ Procedimiento para establecer los rangos de representacin en pantalla}
Procedure Establecer_Rangos_Representacion;
VAR
 MaxX1_aux,MaxX2_aux:REAL;
Begin
 MaxX1_aux:=0;
 MaxX2_aux:=0;
 FOR n:=1 TO numero_restricciones DO
  Begin
   IF MaxX1_aux<Rectas[n].extremo1.x1 Then
    MaxX1_aux:=Rectas[n].extremo1.x1;
   IF MaxX1_aux<Rectas[n].extremo2.x1 Then
    MaxX1_aux:=Rectas[n].extremo2.x1;

   IF MaxX2_aux<Rectas[n].extremo1.x2 Then
    MaxX2_aux:=Rectas[n].extremo1.x2;
   IF MaxX2_aux<Rectas[n].extremo2.x2 Then
    MaxX2_aux:=Rectas[n].extremo2.x2;
  End;

  If MaxX1_aux<>0 Then
   Begin
    MaxX1:=1000000000;
    WHILE MaxX1>MaxX1_aux DO
     MaxX1:=MaxX1/10.00;
    MaxX1:=MaxX1*10;
   End
  Else
   MaxX1:=10;

  If MaxX2_aux<>0 Then
   Begin
    MaxX2:=1000000000;
    WHILE MaxX2>MaxX2_aux DO
     MaxX2:=MaxX2/10.00;
    MaxX2:=MaxX2*10;
   End
  Else
   MaxX2:=10;
End;

{ Procedimiento para recalcular las rectas,las recalcula teniendo en cuenta los
  valores de MaxX1 y MaxX2 }
Procedure Ajustar_rectas;
Begin
 FOR n:=1 TO 3 DO
  If (Pl_aux.restriccion[n].a[1]<>0) AND (Pl_aux.restriccion[n].a[2]=0) Then
   Begin
    rectas[n].extremo1.x2:=0;
    rectas[n].extremo2.x2:=MaxX2;
    If (Pl_aux.restriccion[n].a[1]<>0) Then
     Begin
      rectas[n].extremo1.x1:=Pl_aux.restriccion[n].d/Pl_aux.restriccion[n].a[1];
      rectas[n].extremo2.x1:=rectas[n].extremo1.x1;
      numero_puntos_de_corte:=numero_puntos_de_corte+1;
      puntos_de_corte[numero_puntos_de_corte].x1:=rectas[n].extremo2.x1;
      puntos_de_corte[numero_puntos_de_corte].x2:=MaxX2;
     End;
   End
  Else
   Begin
    rectas[n].extremo2.x1:=MaxX1;
    If (Pl_aux.restriccion[n].a[2]<>0) Then
     Begin
      rectas[n].extremo2.x2:=(Pl_aux.restriccion[n].d-Pl_aux.restriccion[n].a[1]*MaxX1)/Pl_aux.restriccion[n].a[2];
      numero_puntos_de_corte:=numero_puntos_de_corte+1;
      puntos_de_corte[numero_puntos_de_corte].x1:=MaxX1;
      puntos_de_corte[numero_puntos_de_corte].x2:=rectas[n].extremo2.x2;
     End;
   End;
End;

{ Procedimiento para calcular los puntos de corte entre las restricciones }
Procedure calcular_puntos_de_corte_de_rectas;
Var
 x1,x2:Real;
Begin
 { Resolucin sistema de dos ecuaciones dos incognitas }
 {
 En los casos

 0x1+bx2=c  con c=f
 0x1-ex2=f
 y
 ax1+0x2=c  con c=f
 dx1+ex2=f

 las soluciones ya estaran introducidas al calcular el corte de las
 restricciones con los ejes.
 }

 For n:=1 TO numero_restricciones-1 DO
  For m:=n+1 TO numero_restricciones DO
   Begin
    {
    Sistema de ecuaciones a resolver

    ax1+bx2=c
    dx1+ex2=f

    }
    IF (Pl_aux.restriccion[n].a[1]<>0)
       AND
       (Pl_aux.restriccion[n].a[1]*Pl_aux.restriccion[m].a[2]-Pl_aux.restriccion[m].a[1]*Pl_aux.restriccion[n].a[2]<>0) Then
     Begin
      {
      Modo de resolucin

          af-dc
      x2= -----
          db+ae

          c-bx2
      x1= -----
            a
      }

      x2:=(Pl_aux.restriccion[n].a[1]*Pl_aux.restriccion[m].d-Pl_aux.restriccion[m].a[1]*Pl_aux.restriccion[n].d)
          /
          (Pl_aux.restriccion[n].a[1]*Pl_aux.restriccion[m].a[2]-Pl_aux.restriccion[m].a[1]*Pl_aux.restriccion[n].a[2]);
      x1:=(Pl_aux.restriccion[n].d-Pl_aux.restriccion[n].a[2]*x2)/Pl_aux.restriccion[n].a[1];

      numero_puntos_de_corte:=numero_puntos_de_corte+1;
      puntos_de_corte[numero_puntos_de_corte].x1:=x1;
      puntos_de_corte[numero_puntos_de_corte].x2:=x2;
     End
    Else
     If (Pl_aux.restriccion[n].a[2]<>0)
        AND
        (Pl_aux.restriccion[m].a[1]<>0) Then
      Begin
       {
       Modo de resolucin

             c
       x2= -----
             b

           f-ex2
       x1= -----
             d

       }

       x2:=Pl_aux.restriccion[n].d/Pl_aux.restriccion[n].a[2];
       x1:=(Pl_aux.restriccion[m].d-Pl_aux.restriccion[m].a[2]*x1)/Pl_aux.restriccion[m].a[1];

       numero_puntos_de_corte:=numero_puntos_de_corte+1;
       puntos_de_corte[numero_puntos_de_corte].x1:=x1;
       puntos_de_corte[numero_puntos_de_corte].x2:=x2;
     End;
   End;
End;

{ Procedimiento para introducir valores posibles infinitos como puntos de corte }
Procedure Introducir_posibles_resultados_extremos;
Begin
 numero_puntos_de_corte:=numero_puntos_de_corte+1;
 puntos_de_corte[numero_puntos_de_corte].x1:=0;
 puntos_de_corte[numero_puntos_de_corte].x2:=0;

 numero_puntos_de_corte:=numero_puntos_de_corte+1;
 puntos_de_corte[numero_puntos_de_corte].x1:=0;
 puntos_de_corte[numero_puntos_de_corte].x2:=MaxX2;

 numero_puntos_de_corte:=numero_puntos_de_corte+1;
 puntos_de_corte[numero_puntos_de_corte].x1:=MaxX1;
 puntos_de_corte[numero_puntos_de_corte].x2:=0;

 numero_puntos_de_corte:=numero_puntos_de_corte+1;
 puntos_de_corte[numero_puntos_de_corte].x1:=MaxX1;
 puntos_de_corte[numero_puntos_de_corte].x2:=MaxX1;
End;

{ Procedimiento PRINCIPAL }
Procedure Resolucion_grafica;
Begin
 { Clculo del nmero de restricciones y del nmero de variables }
 numero_variables:=0;
 numero_restricciones:=0;
 For n:=1 TO 3 DO
  For m:=1 TO 3 DO
   If (Pl.restriccion[n].a[m]<>0) Then
    Begin
     numero_variables:=m;
     numero_restricciones:=n;
    End
   Else
    If Pl.restriccion[n].d<>0 Then
     numero_restricciones:=n;

 { Numero de variables en funcin objetivo }
 For n:=1 TO 3 DO
  If (Pl.funcion_objetivo.c[n]<>0) AND (n>numero_variables) Then
   numero_variables:=n;

 { Comprobacin de que el programa puede resolverse graficamente }
 If (numero_restricciones=3) AND (numero_variables=3) Then
  Mensaje(1,10,79,15,0,'EL P.L NO PUEDE SER RESUELTO EN 2D.TIENE 3 VARIABLES Y 3 RESTRICCIONES')
 ELSE
  If (numero_variables=3) THEN
   If Pl_aux.primal Then
    Mensaje(1,10,79,15,0,'EL PROBLEMA DEBE SER PASADO A FORMA DUAL.')
   Else
    Mensaje(1,10,79,15,0,'EL PROBLEMA DEBE SER PASADO A FORMA PRIMAL.')
  Else
   Begin
    numero_puntos_de_corte:=0;
    Pl_aux:=Pl;

    { Conversin de la funcin objetivo a forma +MIN o +MAX }
    If Pl_aux.funcion_objetivo.minimizar IN [2,4] Then
     Begin
      If Pl_aux.funcion_objetivo.minimizar=2 Then
       Pl_aux.funcion_objetivo.minimizar:=3
      Else
       Pl_aux.funcion_objetivo.minimizar:=1;
      For n:=1 TO 3 DO
       Pl_aux.funcion_objetivo.c[n]:=-Pl_aux.funcion_objetivo.c[n];
     End;

    { Conversin a forma standard }
    For n:=1 TO numero_restricciones DO
     IF Pl_aux.restriccion[n].IGUALDAD=1 THEN
      BEGIN
       Pl_aux.restriccion[n].IGUALDAD:=0;
       Pl_aux.restriccion[n].a[1]:=-Pl_aux.restriccion[n].a[1];
       Pl_aux.restriccion[n].a[2]:=-Pl_aux.restriccion[n].a[2];
       Pl_aux.restriccion[n].a[3]:=-Pl_aux.restriccion[n].a[3];
       Pl_aux.restriccion[n].d:=-Pl_aux.restriccion[n].d;
      END;

    Establecer_Rectas;
    Calcular_puntos_de_corte_de_rectas;
    Establecer_Rangos_Representacion;
    Ajustar_rectas;
    { Detecto el tipo de tarjeta grfica y la inicializo }
    Inicializar_Graficos;
    GetMaxYY:=GetMaxY-50;{Dejo 50 Pixels para zona de dilogo}
    Introducir_posibles_resultados_extremos;

    { Dibujo de ejes}
    Setcolor(15);
    line(0,0,0,GetMaxYY);
    line(0,GetMaxYY,GetMaxX,GetMaxYY);

    Setcolor(15);
    { Dibujo de rectas }
    For n:=1 TO numero_restricciones DO
     line(ROUND(rectas[n].extremo1.x1*GetMaxX/MaxX1),GetMaxYY-ROUND(rectas[n].extremo1.x2*GetMaxYY/MaxX2),
          ROUND(rectas[n].extremo2.x1*GetMaxX/MaxX1),GetMaxYY-ROUND(rectas[n].extremo2.x2*GetMaxYY/MaxX2));

    { Relleno de soluciones Posibles }
    SetFillStyle(1,7);
    ENCONTRADO:=FALSE;
    i:=0;
    While (i<=MaxX1) AND NOT(encontrado) DO
     Begin
      j:=0;
      While (j<=MaxX2) AND NOT(encontrado) DO
       begin
        IF ((Pl_aux.restriccion[1].a[1]*i+Pl_aux.restriccion[1].a[2]*j)>Pl_aux.restriccion[1].d)
           OR (numero_restricciones<1) THEN
         IF ((Pl_aux.restriccion[2].a[1]*i+Pl_aux.restriccion[2].a[2]*j)>Pl_aux.restriccion[2].d)
            OR (numero_restricciones<2) THEN
          IF ((Pl_aux.restriccion[3].a[1]*i+Pl_aux.restriccion[3].a[2]*j)>Pl_aux.restriccion[3].d)
             OR (numero_restricciones<3) THEN
           begin
            FloodFill(ROUND(i*GetMaxX/MaxX1),GetMaxYY-ROUND(j*GetMaxYY/MaxX2),15);
            If getpixel(ROUND(i*GetMaxX/MaxX1),GetMaxYY-ROUND(j*GetMaxYY/MaxX2))=7 THEN
             Encontrado:=TRUE;
           end;
        j:=j+(MaxX2/100);
       End;
      i:=i+(MaxX1/100);
     End;

    { Clculo del ptimo }
    If encontrado Then
     Begin
      If Pl_aux.funcion_objetivo.minimizar=1 THEN
       MinMax:=MAXINT
      Else
       MinMax:=-Maxint;
      For n:=1 TO numero_puntos_de_corte DO
       Begin
        i:=puntos_de_corte[n].x1;
        j:=puntos_de_corte[n].x2;
        IF (i>=0) AND (j>=0) Then
         IF ((Pl_aux.restriccion[1].a[1]*i+Pl_aux.restriccion[1].a[2]*j)>=Pl_aux.restriccion[1].d)
            OR (numero_restricciones<1) THEN
          IF ((Pl_aux.restriccion[2].a[1]*i+Pl_aux.restriccion[2].a[2]*j)>=Pl_aux.restriccion[2].d)
             OR (numero_restricciones<2) THEN
           IF ((Pl_aux.restriccion[3].a[1]*i+Pl_aux.restriccion[3].a[2]*j)>=Pl_aux.restriccion[3].d)
             OR (numero_restricciones<3) THEN
            IF ((Pl_aux.funcion_objetivo.minimizar=1)
               AND (Pl_aux.funcion_objetivo.c[1]*i+Pl_aux.funcion_objetivo.c[2]*j<MinMax))
             OR
               ((Pl_aux.funcion_objetivo.minimizar=3)
               AND (Pl_aux.funcion_objetivo.c[1]*i+Pl_aux.funcion_objetivo.c[2]*j>MinMax)) Then
             Begin
              OptimoX1:=i;
              OptimoX2:=j;
              MinMax:=Pl_aux.funcion_objetivo.c[1]*i+Pl_aux.funcion_objetivo.c[2]*j;
             End
       End;
     End;

    { Clculo de recta Objetivo }
    CentroPantalla:=Pl_aux.funcion_objetivo.c[1]*MaxX1/2+Pl_aux.funcion_objetivo.c[2]*MaxX2/2;
    If Pl_aux.funcion_objetivo.c[2]<>0 Then
     Begin
      rectas[0].extremo1.x1:=0;
      rectas[0].extremo1.x2:=CentroPantalla/Pl_aux.funcion_objetivo.c[2];
      rectas[0].extremo2.x1:=MaxX1;
      rectas[0].extremo2.x2:=(CentroPantalla-Pl_aux.funcion_objetivo.c[1]*MaxX1)/Pl_aux.funcion_objetivo.c[2];
     End
    Else
     If Pl_aux.funcion_objetivo.c[1]<>0 Then
      Begin
       rectas[0].extremo1.x2:=0;
       rectas[0].extremo1.x1:=CentroPantalla/Pl_aux.funcion_objetivo.c[1];
       rectas[0].extremo2.x2:=MaxX2;
       rectas[0].extremo2.x1:=(CentroPantalla-Pl_aux.funcion_objetivo.c[2]*MaxX2)/Pl_aux.funcion_objetivo.c[1];
      End
     Else
      Begin
       rectas[0].extremo1.x1:=0;
       rectas[0].extremo1.x2:=0;
       rectas[0].extremo2.x1:=MaxX1;
       rectas[0].extremo2.x2:=0;
      End;

    { Dibujo de rectas (restricciones) en diferentes colores }
    SetLineStyle(SolidLn,0,ThickWidth);
    For n:=0 TO numero_restricciones DO
     BEGIN
      Setcolor(color[n]);
      If n=0 Then
       { Evitamos superposiciones }
       line(ROUND(rectas[n].extremo1.x1*GetMaxX/MaxX1)+1,(GetMaxYY-ROUND(rectas[n].extremo1.x2*GetMaxYY/MaxX2)+2),
            ROUND(rectas[n].extremo2.x1*GetMaxX/MaxX1)+1,(GetMaxYY-ROUND(rectas[n].extremo2.x2*GetMaxYY/MaxX2)+2))
      Else
       line(ROUND(rectas[n].extremo1.x1*GetMaxX/MaxX1),GetMaxYY-ROUND(rectas[n].extremo1.x2*GetMaxYY/MaxX2),
            ROUND(rectas[n].extremo2.x1*GetMaxX/MaxX1),GetMaxYY-ROUND(rectas[n].extremo2.x2*GetMaxYY/MaxX2));
     END;
    SetLineStyle(SolidLn,0,NormWidth);

    { Dibujo del ptimo }
    If encontrado AND (OptimoX1<>MaxX1) AND (OptimoX2<>MaxX2) Then
     Begin
      Es_optimo_recta:=FALSE;
      Setcolor(color[4]);
      For n:=1 TO numero_puntos_de_corte DO
       Begin
        i:=puntos_de_corte[n].x1;
        j:=puntos_de_corte[n].x2;
        If (i>=0) AND (j>=0) Then
         IF ((Pl_aux.restriccion[1].a[1]*i+Pl_aux.restriccion[1].a[2]*j)>=Pl_aux.restriccion[1].d)
            OR (numero_restricciones<1) THEN
          IF ((Pl_aux.restriccion[2].a[1]*i+Pl_aux.restriccion[2].a[2]*j)>=Pl_aux.restriccion[2].d)
             OR (numero_restricciones<2) THEN
           IF ((Pl_aux.restriccion[3].a[1]*i+Pl_aux.restriccion[3].a[2]*j)>=Pl_aux.restriccion[3].d)
             OR (numero_restricciones<3) THEN
            IF (Pl_aux.funcion_objetivo.c[1]*i+Pl_aux.funcion_objetivo.c[2]*j=MinMax)
               AND
               ((i<>OptimoX1) OR (j<>OptimoX2))  Then
             Begin
              SetLineStyle(UserBitLn,$FF,ThickWidth);
              line(ROUND(OptimoX1*GetMaxX/MaxX1),GetMaxYY-ROUND(OptimoX2*GetMaxYY/MaxX2),
                   ROUND(i*GetMaxX/MaxX1),GetMaxYY-ROUND(j*GetMaxYY/MaxX2));
              SetLineStyle(SolidLn,0,NormWidth);
              Es_optimo_recta:=TRUE;
             End
       End;
      If NOT(Es_optimo_recta) Then
       Begin
        SetFillStyle(1,color[4]);
        circle(ROUND(OptimoX1*GetMaxX/MaxX1),GetMaxYY-ROUND(OptimoX2*GetMaxYY/MaxX2),4);
        FloodFill(ROUND(OptimoX1*GetMaxX/MaxX1),GetMaxYY-ROUND(OptimoX2*GetMaxYY/MaxX2),color[4]);
       End;
     End;

    { Borrado de zona de dilogo }
    SetFillStyle(1,0);
    BAR(0,GetMaxYY,GetmaxX,GetMaxY);

    { Escritura en zona de dilogo }
    { Escritura restricciones }
    For n:=1 TO numero_restricciones DO
     BEGIN
      Setcolor(color[n]);
      STR(n:1,cadena_aux);
      OUTTEXTXY(10,GetMaxYY+12*n,'RESTRICCION '+cadena_aux);
     END;

    { Escritura soluciones posibles }
    Setcolor(15);
    SetFillStyle(1,7);
    BAR(155,GetMaxYY+5,170,GetMaxYY+18);
    rectangle(155,GetMaxYY+5,170,GetMaxYY+18);
    OUTTEXTXY(175,GetMaxYY+12,'SOLUCIONES POSIBLES');

    { Escritura funcin objetivo }
    SetColor(color[0]);
    OUTTEXTXY(175,GetMaxYY+24,'FUNCION OBJETIVO');

    { Escritura del ptimo}
    SetColor(color[4]);
    { Dibujo un punto o una recta en la zona de dilogo segn si la
      solucin es una recta o no }
    If Es_optimo_recta Then
     Begin
      SetLineStyle(UserBitln,$FF,ThickWidth);
      line(385,GetMaxYY+15,397,GetMaxYY+15);
      SetLineStyle(SolidLn,0,NormWidth);
     End
    Else
     Begin
      SetFillStyle(1,color[4]);
      CIRCLE(390,GetMaxYY+15,3);
      FloodFill(390,GetMaxYY+15,color[4]);
     End;
    OUTTEXTXY(400,GetMaxYY+12,'OPTIMO');
    IF encontrado Then
     If NOT(Es_optimo_recta) Then
      Begin
       { Escritura en zona de dilogo de los datos del ptimo }
       IF GetMaxX-ROUND(OptimoX1*GetMaxX/MaxX1)<0.5*MaxX1 Then
        Begin
         OUTTEXTXY(400,GetMaxYY+24,'    X1= INFINITO');
         MinMax:=MAXINT;
        End
       Else
        BEGIN
         STR(OptimoX1:10:4,cadena_aux);
         OUTTEXTXY(400,GetMaxYY+24,'    X1='+cadena_aux);
        END;
       If GetMaxYY-ROUND(OptimoX2*GetMaxYY/MaxX2)<0.5*MaxX2 Then
        Begin
         OUTTEXTXY(400,GetMaxYY+36,'    X2= INFINITO');
         MinMax:=MAXINT;
        End
       Else
        BEGIN
         STR(OptimoX2:10:4,cadena_aux);
         OUTTEXTXY(400,GetMaxYY+36,'    X2='+cadena_aux);
        END;
       If MinMax=Maxint Then
        OUTTEXTXY(400,GetMaxYY+12,'OPTIMO= INFINITO')
       Else
        Begin
         STR(MinMax:10:4,cadena_aux);
         OUTTEXTXY(400,GetMaxYY+12,'OPTIMO='+cadena_aux);
        End;
      End
     Else
       OUTTEXTXY(400,GetMaxYY+24,'EXISTEN MUCHAS SOLUCIONES')
    Else
     OUTTEXTXY(400,GetMaxYY+24,'NO EXISTE');

    { Dibujo de ejes }
    Setcolor(15);
    line(0,0,0,GetMaxYY);
    line(0,GetMaxYY,GetMaxX,GetMaxYY);
    { Numeracin de ejes }
    OUTTEXTXY(5,GetMaxYY-10,'0.0');
    PUTPIXEL(1,GetMaxYY-1,15);
    PUTPIXEL(2,GetMaxYY-2,15);
    FOR n:=1 TO 9 DO
     Begin
      STR((MaxX1*n/10):5:4,cadena_aux);
      OUTTEXTXY(n*ROUND(GetMaxX/10),GetMaxYY-(10*(n MOD 2+1)),cadena_aux);
      PUTPIXEL(n*ROUND(GetMaxX/10),GetMaxYY-1,15);
      PUTPIXEL(n*ROUND(GetMaxX/10),GetMaxYY-2,15);
      STR((MaxX2*n/10):5:4,cadena_aux);
      OUTTEXTXY(5,GetMaxYY-(n*ROUND(GetMaxYY/10)),cadena_aux);
      PUTPIXEL(1,GetMaxYY-(n*ROUND(GetMaxYY/10)),15);
      PUTPIXEL(2,GetMaxYY-(n*ROUND(GetMaxYY/10)),15);
     End;
    OUTTEXTXY(5,10,'X2');
    OUTTEXTXY(GetMaxX-20,GetMaxYY+8,'X1');


    { Espero hasta que se pulse una tecla }
    tecla:=READKEY;
    tecla:=' ';

    { Vuelvo al modo texto de la pantalla }
    CloseGraph;
   End;
End;

BEGIN
END.